/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.client.internal.ContributorInfoRegistry");

dojo.require("com.ibm.team.dashboard.web.client.internal.Constants");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

(function() {

/*
 * Aliases
 */
var Constants = com.ibm.team.dashboard.web.client.internal.Constants;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;

/*
 * A client-side data caching layer for dashboard contributor info.
 */
com.ibm.team.dashboard.web.client.internal.ContributorInfoRegistry = {
	
	_infoByDashboardItemId: new Object(),
	
	/*
	 * Adds the given info to the registry. Contributor info is bundled
	 * with other dashboard requests, e.g. getDashboard.
	 */
	add: function(/* DashboardContributorInfo */ info) {
		this._infoByDashboardItemId[info.dashboard.itemId] = info;
		// convert to associative array for fast lookup
		var viewletInfoByViewletId = new Object();
		var viewletInfo = info.viewletInfo;
		if (viewletInfo) {
			for (var i=0;i<viewletInfo.length;i++) {
				viewletInfoByViewletId[viewletInfo[i].viewletId] = viewletInfo[i];
			}
		}
		info.viewletInfo = viewletInfoByViewletId;
	},

	/*
	 * Returns the contributor memento (state) for the given viewlet; creates one
	 * if none exists (but it is not saved yet).
	 */	
	/* string */ getMemento: function(/* string */ dashboardItemId, /* string */ viewletId) {
		var viewletInfo = this._getViewletInfo(dashboardItemId, viewletId);
		if (viewletInfo) {
			return viewletInfo.memento;
		}
	},
	
	/*
	 * Returns whether or not the given viewlet is collapsed.
	 */
	/* boolean */ isCollapsed: function(/* string */ dashboardItemId, /* string */ viewletId) {
		var viewletInfo = this._getViewletInfo(dashboardItemId, viewletId, false);
		if (viewletInfo) {
			// bug 31776: REST service not mapping EBoolean to boolean.
			var value = viewletInfo.collapsed;
			if (!value) {
				// default is not collapsed
				value = false;
			}
			else if (typeof value  == "string") {
				value = (value  == "true");
			}
			return value;
		}
		// default is not collapsed
		return false;
	},
	
	/*
	 * Sets whether or not the given viewlet is collapsed.
	 */	
	setCollapsed: function(/* string */ dashboardItemId, /* string */ viewletId, /* boolean */ value) {
		this._getViewletInfo(dashboardItemId, viewletId, true).collapsed = value;
		var srh = new ServiceResponseHandler(this, "_success", "_error");
		var req = new ServiceRequest(Constants.SERVICE_URI, "postViewletCollapsed", {dashboardItemId: dashboardItemId, viewletId: viewletId, value: value});
		TeamServerClient.invokeService(req, srh);
	},

	/*
	 * Sets the contributor memento for the given viewlet.
	 */	
	setMemento: function(/* string */ dashboardItemId, /* string */ viewletId, /* string */ memento) {
		this._getViewletInfo(dashboardItemId, viewletId, true).memento = memento;
		var srh = new ServiceResponseHandler(this, "_success", "_error");
		var req = new ServiceRequest(Constants.SERVICE_URI, "postViewletContributorMemento", {dashboardItemId: dashboardItemId, viewletId: viewletId, memento: memento});
		TeamServerClient.invokeService(req, srh);
	},
	
	/*
	 * The service error callback.
	 */
	_error: function(/* Error */ error) {
		dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
	},

	/*
	 * Returns the contributor info for the given dashboard. If create is true, will create
	 * the necessary objects (not saved on server yet).
	 */	
	/* DashboardContributorInfo */ _getDashboardInfo: function(/* string */ dashboardItemId, /* boolean */ create) {
		var dashboardInfo = this._infoByDashboardItemId[dashboardItemId];
		if (!dashboardInfo && create) {
			dashboardInfo = new Object();
			this._infoByDashboardItemId[dashboardItemId] = dashboardInfo;
		}
		return dashboardInfo;
	},
	
	/*
	 * Returns the contributor info for the given viewlet. If create is true, will create
	 * the necessary objects (not saved on server yet).
	 */
	/* ViewletContributorInfo */ _getViewletInfo: function(/* string */ dashboardItemId, /* string */ viewletId, /* boolean */ create) {
		var dashboardInfo = this._getDashboardInfo(dashboardItemId, create);
		if (dashboardInfo) {
			var viewletInfos = dashboardInfo.viewletInfo;
			if (!viewletInfos && create) {
				viewletInfos = new Object();
				dashboardInfo.viewletInfo = viewletInfos;
			}
			if (viewletInfos) {
				var viewletInfo = dashboardInfo.viewletInfo[viewletId];
				if (!viewletInfo && create) {
					viewletInfo = new Object();
					dashboardInfo.viewletInfo[viewletId] = viewletInfo;
				}
				return viewletInfo;
			}
		}
	},
	
	/*
	 * The service success callback.
	 */
	_success: function() {
		// do nothing
	}
};

})();
